-- AddOn by Dezyne, Argent Dawn, Europe. Please give me credit when you want to redistribute or modify this addon! Contact: coenvdwel@planet.nl
													 local rewatch_version = 1.1; -- current revision
--// FUNCTIONS //----------------------------------------------------------------------------------------------------------------------------------------------------------------

-- trigger a cooldown overlay
-- spellName: the name of the spell to trigger the overlay from
-- return: void
function rewatch_TriggerCooldown(spellName)
	-- only get cooldown of the spell if there's a button for it
	if(rewatch_abs[spellName]["Button"]) then
		-- get the cooldown on the spell
		local start, duration, enabled = GetSpellCooldown(spellName);
		-- check if the spell even exists (non-resto, or low leveled)
		if(start == nil) then
			-- hide the button
			rewatch_abs[spellName]["Button"]:Hide();
		else
			-- set the cooldown frame timer on the cooldown overlay of the spell's button
			CooldownFrame_SetTimer(rewatch_abs[spellName]["Button"].cooldown, start, duration, enabled);
		end;
	end;
	
	-- return
	return;
end;

-- adjusts the parent frame container's height
-- d: the change in height to be processed; negative for shrinking, positive for enlarging
-- return: void
function rewatch_AlterFrame(d)
	-- alter the height directly
	rewatch_f:SetHeight(rewatch_f:GetHeight() + d);
	
	-- return
	return;
end;

-- checks if the player is in the group
-- player: name of the player to check for
-- return: true, if the player is the user, or in the user's party or raid; false elsewise
function rewatch_InGroup(player)
	-- initialise local vars
	local env, pos, n = "", 1, 1;
	-- catch a self-check; return true if searching for the user itself
	if(UnitName("player") == player) then return true; end;
	-- set environment (party or raid)
	if(GetNumRaidMembers() > 0) then env, n = "raid", GetNumRaidMembers();
	elseif(GetNumPartyMembers() > 0) then env, n = "party", GetNumPartyMembers();
	-- if no environment found, the player cannot be grouped with the user
	else return false; end;
	-- for every possible spot in the environment...
	while(pos <= n) do
		-- check if the names match, return true when found
		if(UnitName(env..pos) == player) then return true; end;
		pos = pos + 1;
	end;
	
	-- return false if not found
	return false;
end;

-- display a message to the user in the chat pane
-- msg: the message to pass onto the user
-- return: void
function rewatch_Message(msg)
	-- send the message to the chat pane, in yellow-ish
	DEFAULT_CHAT_FRAME:AddMessage(msg, 0.95, 0.85, 0.15);
	
	-- return
	return;
end;

-- round a number to the supplied number of decimals
-- num: the number to round
-- dec: the number of decimals to show
-- return: floor(num * 10^dec + 0.5) / 10^dec
function rewatch_Round(num, dec)
	-- default the dec to 0 and create multiply factor
    local mult = 10 ^ (dec or 0);
	
	-- return
    return math.floor(num * mult + 0.5) / mult;
end;

-- get the number of the supplied player's place in the player table, or -1
-- player: name of the player to search for
-- return: the supplied player's table index, or -1 if not found
function rewatch_GetPlayer(player)
	-- for every seen player
	for i,val in ipairs(rewatch_bars) do
		-- return if the name matches the supplied name
		if(val["Player"] == player) then return i; end;
	end;
	
	-- return -1 if not found
	return -1;
end;

-- add a player to the players table and create his bars and button
-- player: the name of the player
-- return: the index number the player has been assigned
function rewatch_AddPlayer(player)
	-- create heading
	rewatch_bars[rewatch_i] = {}; rewatch_AlterFrame(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]);
	local statusbar = CreateFrame("STATUSBAR", nil, rewatch_f, "TextStatusBar"); statusbar:SetWidth(rewatch_data["SpellBarWidth"]); statusbar:SetHeight(rewatch_data["SpellBarHeight"]); statusbar:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_data["SpellBarMargin"]*2, 25-(rewatch_i+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
	statusbar:SetBackdrop({bgFile = nil, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = 1, tileSize = 5, edgeSize = 5, insets = { left = 0, right = 0, top = 0, bottom = 0 }});
	statusbar:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar"); statusbar:SetStatusBarColor(0.0, 0.6, 0.0, 0.8);
	statusbar:SetMinMaxValues(0, 1); statusbar:SetValue(0);
	-- put text in heading
	local statusbarText = statusbar:CreateFontString("$parentText", "ARTWORK", "GameFontHighlightSmall");
	statusbarText:SetPoint("LEFT", statusbar); statusbarText:SetAllPoints(); statusbarText:SetText(player);
	-- overlay target/remove button
	local tgb = CreateFrame("BUTTON", nil, statusbar, "SecureActionButtonTemplate");
	tgb:SetWidth(statusbar:GetWidth()); tgb:SetHeight(statusbar:GetHeight()); tgb:SetPoint("TOPLEFT", statusbar, "TOPLEFT", 0, 0);
	tgb:RegisterForClicks("LeftButtonDown"); tgb:SetAttribute("type1", "target"); tgb:SetAttribute("unit", player);
	tgb:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square.blp");
	tgb:SetScript("OnMouseDown", function() if(arg1 == "RightButton") then rewatch_HidePlayerByName(player); end; end);
	-- add a swiftmend button
	local smb = CreateFrame("BUTTON", nil, rewatch_f, "SecureActionButtonTemplate");
	smb:SetWidth(rewatch_data["SmallButtonWidth"]); smb:SetHeight(rewatch_data["SmallButtonHeight"]); smb:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs["Swiftmend"]["Offset"], 25-(rewatch_i+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
	smb:RegisterForClicks("LeftButtonDown"); smb:SetAttribute("type1", "spell"); smb:SetAttribute("unit", player);
	smb:SetAttribute("spell1", "Swiftmend"); smb:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square.blp");
	smb:SetNormalTexture("Interface\\Icons\\Spell_Nature_NatureBlessing.blp");
	-- create the cooldown overlay
	local smbcd = CreateFrame("Cooldown", nil, smb, "CooldownFrameTemplate");
	smb.cooldown = smbcd; smbcd:SetPoint("CENTER", 0, -1);
	smbcd:SetWidth(22); smbcd:SetHeight(22); smbcd:Hide();
	-- add abolish poison button
	local apb = CreateFrame("BUTTON", nil, rewatch_f, "SecureActionButtonTemplate");
	apb:SetWidth(rewatch_data["SmallButtonWidth"]); apb:SetHeight(rewatch_data["SmallButtonHeight"]); apb:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs["AbolishPoison"]["Offset"], 25-(rewatch_i+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
	apb:RegisterForClicks("LeftButtonDown"); apb:SetAttribute("type1", "spell"); apb:SetAttribute("unit", player);
	apb:SetAttribute("spell1", "Abolish Poison"); apb:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square.blp");
	apb:SetNormalTexture("Interface\\Icons\\Spell_Nature_NullifyPoison_02.blp");
	-- save player data
	rewatch_bars[rewatch_i]["Player"] = player; rewatch_bars[rewatch_i]["PlayerBar"] = statusbar;
	rewatch_bars[rewatch_i]["Rejuvenation"] = 0; rewatch_bars[rewatch_i]["Regrowth"] = 0;
	rewatch_bars[rewatch_i]["RejuvenationBar"] = rewatch_CreateBar("Rejuvenation", rewatch_i);
	rewatch_bars[rewatch_i]["RegrowthBar"] = rewatch_CreateBar("Regrowth", rewatch_i);
	rewatch_bars[rewatch_i]["SwiftmendButton"] = smb; rewatch_bars[rewatch_i]["AbolishPoisonButton"] = apb;
	-- increment the global index
	rewatch_i = rewatch_i+1;
	
	-- sort the players table accordingly
	rewatch_SortPlayers();
	
	-- return the inserted player's player table index
	return rewatch_GetPlayer(player);
end;

-- hide all bars and buttons from - and the player himself
-- playerId: the table index of the player to hide
-- return: void
function rewatch_HidePlayer(playerId)
	-- if this action would decrease the number of visible bars, shrink parent frame
	if(rewatch_bars[playerId]["PlayerBar"]:IsVisible()) then
		rewatch_AlterFrame(-1 * (rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
	end;
	-- hide the bars
	rewatch_bars[playerId]["RejuvenationBar"]:Hide(); rewatch_bars[playerId]["RegrowthBar"]:Hide();
	rewatch_bars[playerId]["PlayerBar"]:Hide(); rewatch_bars[playerId]["SwiftmendButton"]:Hide();
	rewatch_bars[playerId]["AbolishPoisonButton"]:Hide();
	
	-- sort the players table accordingly
	rewatch_SortPlayers();
	
	-- return
	return;
end;

-- hide all bars and buttons from - and the player himself, by name
-- player: the name of the player to hide
-- return: void
function rewatch_HidePlayerByName(player)
	-- get the index of this player
	local playerId = rewatch_GetPlayer(player);
	-- if this player exists, hide all bars and buttons from - and the player himself
	if(playerId > 0) then rewatch_HidePlayer(playerId); end;
	
	-- return
	return;
end;

-- create a pressable, textured  button with a cooldown overlay
-- spellName: the name of the spell to create a button for (must be in rewatch_abs and have Offset
-- icon: the name of the icon to be used as texture for the button
-- return: the reference to the button
function rewatch_CreateButton(spellName, icon)
	-- create the button
	local btn = CreateFrame("BUTTON", nil, rewatch_f, "SecureActionButtonTemplate");
	btn:SetWidth(rewatch_data["ButtonWidth"]); btn:SetHeight(rewatch_data["ButtonHeight"]); btn:SetPoint("BOTTOMRIGHT", rewatch_f, "BOTTOMRIGHT", rewatch_abs[spellName]["Offset"], 5);
	btn:RegisterForClicks("LeftButtonDown"); btn:SetAttribute("type1", "spell");
	btn:SetAttribute("spell1", spellName); btn:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square.blp");
	btn:SetNormalTexture("Interface\\Icons\\"..icon);
	-- create the button overlay
	local btncd = CreateFrame("Cooldown", nil, btn, "CooldownFrameTemplate");
	btn.cooldown = btncd; btncd:SetPoint("CENTER", 0, -1);
	btncd:SetWidth(rewatch_data["ButtonWidth"]); btncd:SetHeight(rewatch_data["ButtonHeight"]); btncd:Hide();
	-- add the button reference to the rewatch_abs table
	rewatch_abs[spellName]["Button"] = btn;
	
	-- return the reference to the button
	return btn;
end;

-- create a spell bar with text and add it to the global player table
-- spellName: the name of the spell to create a bar for (must be in rewatch_abs and have Offset)
-- playerId: the index number of the player in the player table
-- return: the created spell bar reference
function rewatch_CreateBar(spellName, playerId)
	-- create the bar
	local b = CreateFrame("STATUSBAR", spellName..playerId, rewatch_f, "TextStatusBar"); b:SetWidth(rewatch_data["SpellBarWidth"]); b:SetHeight(rewatch_data["SpellBarHeight"]); b:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[spellName]["Offset"],  25-(playerId+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
	b:SetBackdrop({bgFile = nil, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 5, edgeSize = 5, insets = { left = 0, right = 0, top = 0, bottom = 0 }});
	b:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar"); b:SetStatusBarColor(0.0, 0.0, 0.6, 0.8);
	b:SetMinMaxValues(0, rewatch_abs[spellName]["Duration"]); b:SetValue(0);
	-- put text in status bar
	local bs = b:CreateFontString("$parentText", "ARTWORK", "GameFontHighlightSmall");
	bs:SetPoint("RIGHT", b); bs:SetAllPoints(); bs:SetText(spellName);
	-- overlay cast button
	local bc = CreateFrame("BUTTON", nil, b, "SecureActionButtonTemplate");
	bc:SetWidth(100); bc:SetHeight(14); bc:SetPoint("TOPLEFT", b, "TOPLEFT", 0, 0);
	bc:RegisterForClicks("LeftButtonDown"); bc:SetAttribute("type1", "spell"); bc:SetAttribute("unit", rewatch_bars[playerId]["Player"]);
	bc:SetAttribute("spell1", spellName); bc:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square.blp");
	
	-- return the reference to the spell bar
	return b;
end;

-- update a bar by resetting spell duration
-- spellName: the name of the spell to reset it's duration from
-- playerId: the index number of the player in the player table
-- return: void
function rewatch_UpdateBar(spellName, playerId)
	-- if the player's bar is not visible, make it reappear
	if(not rewatch_bars[playerId]["PlayerBar"]:IsVisible()) then
		-- show all bars
		rewatch_bars[playerId]["PlayerBar"]:Show();
		rewatch_bars[playerId]["RejuvenationBar"]:Show();
		rewatch_bars[playerId]["RegrowthBar"]:Show();
		rewatch_bars[playerId]["SwiftmendButton"]:Show();
		rewatch_bars[playerId]["AbolishPoisonButton"]:Show();
		-- refresh the end time and bar value
		rewatch_bars[playerId][spellName] = GetTime() +  rewatch_abs[spellName]["Duration"];
		rewatch_bars[playerId][spellName.."Bar"]:SetValue(rewatch_abs[spellName]["Duration"]);
		-- increase the parent container's height
		rewatch_AlterFrame(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]);
		-- sort the players again
		rewatch_SortPlayers();
	else
		-- refresh the end time and bar value
		rewatch_bars[playerId][spellName] = GetTime() +  rewatch_abs[spellName]["Duration"];
		rewatch_bars[playerId][spellName.."Bar"]:SetValue(rewatch_abs[spellName]["Duration"]);
	end;
	
	-- return
	return;
end;

-- process a swiftmend cast by the player, on specified playerId
-- playerId: the index number of the player in the player table
-- return: void
function rewatch_ProcessSwiftmend(playerId)
	-- assume it consumed a rejuvenation
	local consumed = "Rejuvenation";
	-- check if this assumption was wrong, and if so, alter it
	if((rewatch_bars[playerId]["Rejuvenation"] == 0) or (((rewatch_bars[playerId]["Regrowth"] > 0)) and (rewatch_bars[playerId]["Regrowth"] < rewatch_bars[playerId]["Rejuvenation"]))) then
		consumed = "Regrowth";
	end;
	-- clear the bar that was consumed and unset it's end time
	rewatch_bars[playerId][consumed] = 0; rewatch_bars[playerId][consumed.."Bar"]:SetValue(0);
	-- trigger all cooldown overlays of every player's swiftmend button
	for i,val in ipairs(rewatch_bars) do CooldownFrame_SetTimer(val["SwiftmendButton"].cooldown, GetTime(), rewatch_abs["Swiftmend"]["Duration"], 1); end;
	-- also hide an ungrouped player when his hots run out because of a SM
	if((rewatch_bars[playerId]["Rejuvenation"] == 0) and (rewatch_bars[playerId]["Regrowth"] == 0)) then if(not rewatch_InGroup(rewatch_bars[playerId]["Player"])) then rewatch_HidePlayer(playerId); end; end;
	
	-- return
	return;
end;

-- sort the players into a solid lineup (no gaps in between)
-- return: void
function rewatch_SortPlayers()
	-- initialise local vars and repeat as long as there's a swap
	local swap, n, m = true, 1, 2; while(swap) do
		-- reset the swap bool
		swap = false;
		-- find a hidden bar which causes a gap
		while(rewatch_bars[n+1]) do
			if(rewatch_bars[n]["PlayerBar"]:IsVisible()) then n = n + 1;
			else break; end;
		end; m = n+1;
		-- find the next visible bar that could fill this gap
		while(rewatch_bars[m]) do
			if(not rewatch_bars[m]["PlayerBar"]:IsVisible()) then m = m + 1;
			else break; end;
		end;
		-- check the found results; they may also be the last+1 index of the table
		if((rewatch_bars[n]) and (rewatch_bars[m])) then
			if((rewatch_bars[m]["PlayerBar"]:IsVisible()) and (not rewatch_bars[n]["PlayerBar"]:IsVisible())) then
				-- swap the found table values
				swap = true; local temp = rewatch_bars[n];
				rewatch_bars[n] = rewatch_bars[m];
				rewatch_bars[n]["PlayerBar"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_data["SpellBarMargin"]*2, 25-(n+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[n]["RejuvenationBar"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs["Rejuvenation"]["Offset"], 25-(n+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[n]["RegrowthBar"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs["Regrowth"]["Offset"], 25-(n+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[n]["SwiftmendButton"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs["Swiftmend"]["Offset"], 25-(n+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[n]["AbolishPoisonButton"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs["AbolishPoison"]["Offset"], 25-(n+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[m] = temp;
				rewatch_bars[m]["PlayerBar"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_data["SpellBarMargin"]*2, 25-(m+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[m]["RejuvenationBar"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs["Rejuvenation"]["Offset"], 25-(m+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[m]["RegrowthBar"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs["Regrowth"]["Offset"], 25-(m+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[m]["SwiftmendButton"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs["Swiftmend"]["Offset"], 25-(m+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[m]["AbolishPoisonButton"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs["AbolishPoison"]["Offset"], 25-(m+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
			end;
		end;
	end; -- will end when there's no more swap done
	
	-- return
	return;
end;

-- performs a poison check on given unit, will return true if a poison debuff was found
-- playerId: the index number of the player in the player table
-- return: true if a poison debuff was found, false if not
function rewatch_IsPoisoned(playerId)
	-- go through all possible debuff spots
	local debuffType; for i=1,40 do
		-- get the debuff type, return true if it's poison
		debuffType = select(5, UnitDebuff(rewatch_bars[playerId]["Player"], i, 1));
		if(debuffType == "Poison") then return true;
		-- or, if it's empty, we reached the end of the debuff queue, return false
		elseif((debuffType == "") or (not debuffType)) then return false end;
	end;
	
	-- return
	return false;
end;

-- process the sent commands
-- cmd: the command that has to be executed
-- return: void
function rewatch_SlashCommandHandler(cmd)
	-- if the frame is visible
	if(rewatch_f:IsVisible()) then
		-- display credits
		rewatch_Message("Rewatch v"..rewatch_version.." addon by Dezyne (AD EU). No console commands are supported.");
	-- else toggle the frame back to visible
	else rewatch_f:Show(); end;
	
	-- return
	return;
end;


--// INITIALISE //----------------------------------------------------------------------------------------------------------------------------------------------------------------

-- make the addon stop here if the user isn't a druid
if((select(2, UnitClass("player"))) ~= "DRUID") then return; end;
-- initialise all vars
rewatch_f, rewatch_i, rewatch_bars = CreateFrame("FRAME", "Rewatch_Frame", UIParent),  1, {};
rewatch_update, btnI, btnNS, btnT, btnMove, btnBS, btnExit, btnGotW = nil, nil, nil, nil, nil, nil, nil, nil;
-- add the slash command handler
SLASH_REWATCH1 = "/rewatch";
SlashCmdList["REWATCH"] = function(cmd)
	rewatch_SlashCommandHandler(cmd);
end;
-- initialise the data table
rewatch_data = {
	FrameWidth = 353; FrameHeight = 40;
	SpellBarWidth = 100; SpellBarHeight = 14; SpellBarMargin = 3;
	SmallButtonWidth = 14; SmallButtonHeight = 14; SmallButtonMargin = 3;
	ButtonWidth = 22; ButtonHeight = 22; ButtonMargin = 3;
};
-- initialise the abilities table
rewatch_abs = {
	["Rejuvenation"] = {
		Duration = 12;
		Offset = rewatch_data["SpellBarMargin"]*2+(rewatch_data["SpellBarWidth"]+rewatch_data["SpellBarMargin"])*1;
	};
	["Regrowth"] = {
		Duration = 21;
		Offset = rewatch_data["SpellBarMargin"]*2+(rewatch_data["SpellBarWidth"]+rewatch_data["SpellBarMargin"])*2;
	};
	["Swiftmend"] = {
		Duration = 15;
		Offset = rewatch_data["SpellBarMargin"]*2+(rewatch_data["SpellBarWidth"]+rewatch_data["SpellBarMargin"])*3;
	};
	["AbolishPoison"] = {
		Duration = 1.5;
		Offset = rewatch_data["SpellBarMargin"]*2+(rewatch_data["SpellBarWidth"]+rewatch_data["SpellBarMargin"])*3+rewatch_data["SmallButtonWidth"]+rewatch_data["SmallButtonMargin"];
	};
	["Barkskin"] = {
		Button = btnBS;
		Offset = -30-(rewatch_data["ButtonWidth"]+rewatch_data["ButtonMargin"])*2;
	};
	["Nature's Swiftness"] = {
		Button = btnNS;
		Offset = -30;
	};
	["Innervate"] = {
		Button = btnI;
		Offset = -30-(rewatch_data["ButtonWidth"]+rewatch_data["ButtonMargin"])*1;
	};
	["Tranquility"] = {
		Button = btnT;
		Offset = -30-(rewatch_data["ButtonWidth"]+rewatch_data["ButtonMargin"])*3;
	};
	["Gift of the Wild"] = {
		Button = btnGotW;
		Offset = -30-(rewatch_data["ButtonWidth"]+rewatch_data["ButtonMargin"])*4;
	};
};


--// SCRIPT //-----------------------------------------------------------------------------------------------------------------------------------------------------------------------

-- create the outline frame
rewatch_f:SetWidth(rewatch_data["FrameWidth"]); rewatch_f:SetHeight(rewatch_data["FrameHeight"]);
rewatch_f:SetPoint("TOPLEFT", UIParent, "TOPLEFT"); rewatch_f:EnableMouse(true); rewatch_f:SetMovable(true);
rewatch_f:SetBackdrop({bgFile = nil, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = 1, tileSize = 10, edgeSize = 10, insets = { left = 3, right = 3, top = 3, bottom = 3 }});
rewatch_f:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
rewatch_f:SetScript("OnMouseDown", function() if(btnMove:GetChecked()) then rewatch_f:StartMoving(); moving = true; end; end);
rewatch_f:SetScript("OnMouseUp", function() if(moving) then rewatch_f:StopMovingOrSizing(); moving = false; end; end);
-- make sure this frame catches combat_log_event_unfiltered events and processes them
rewatch_f:SetScript("OnEvent", function(timestamp, event, ...)
	-- if an aura has been removed from the user
	if(arg2 == "SPELL_AURA_REMOVED") then if(arg6 == UnitGUID("player")) then
			-- if the Nature's Swiftness aura has been removed from the user itself, trigger the cooldown overlay
			if(arg10 == "Nature's Swiftness") then rewatch_update = arg10; end;
		end;
	-- if a spell was cast successfull by the yser or a heal has been received
	elseif((arg2 == "SPELL_CAST_SUCCESS") or (arg2 == "SPELL_HEAL")) then if(arg3 == UnitGUID("player")) then
		-- if this was a regrowth or rejuvenation HoT effect
		if((arg10 == "Regrowth") or (arg10 == "Rejuvenation")) then
			-- get the player's table index number
			local playerId = rewatch_GetPlayer(arg7);
			-- if this player doesn't exist in the table yet, add him to it
			if(playerId < 0) then playerId = rewatch_AddPlayer(arg7); end;
			-- update the bar of the corresponding spell and player
			rewatch_UpdateBar(arg10, playerId);
		-- if a swiftmend was received (bugfix: spell launches both events, want to catch only one)
		elseif((arg10 == "Swiftmend") and (arg2 == "SPELL_HEAL")) then
			-- get the player's table index number
			local playerId = rewatch_GetPlayer(arg7);
			-- process the cast swiftmend
			rewatch_ProcessSwiftmend(playerId);
		-- if it's a different spell that has a button on the frame, trigger it's cooldown overlay
		elseif((arg10 == "Innervate") or (arg10 == "Barkskin") or (arg10 == "Tranquility")) then rewatch_update = arg10; end;
	end; end;
	
	-- return
	return;
end);

-- build checkbutton
btnMove = CreateFrame("CHECKBUTTON", "btnMove", rewatch_f, "OptionsCheckButtonTemplate");
btnMove:SetPoint("BOTTOMRIGHT", rewatch_f, "BOTTOMRIGHT", 0, 0); btnMove:SetChecked(false);

-- build exit button
btnExit = CreateFrame("BUTTON", "btnExit", rewatch_f, "OptionsButtonTemplate");
btnExit:SetPoint("BOTTOMLEFT", rewatch_f, "BOTTOMLEFT", 5, 5); btnExit:SetText("Close");
btnExit:SetScript("OnClick", function() rewatch_f:Hide(); rewatch_Message("Rewatch addon frame hidden. Type \"/rewatch\" to toggle it again."); end);

-- build spell buttons
btnNS = rewatch_CreateButton("Nature's Swiftness", "Spell_Nature_RavenForm.blp");
btnI = rewatch_CreateButton("Innervate", "Spell_Nature_Lightning.blp");
btnBS = rewatch_CreateButton("Barkskin", "Spell_Nature_StoneClawTotem.blp");
btnT = rewatch_CreateButton("Tranquility", "Spell_Nature_Tranquility.blp");
btnGotW = rewatch_CreateButton("Gift of the Wild", "Spell_Nature_GiftoftheWild.blp");

-- have the statusbar simulate slice and dice timeleft
rewatch_f:SetScript("OnUpdate", function()
	-- if the update var hasn't been initialised; onload
	if(rewatch_update == nil) then
		-- clear the update var
		rewatch_update = "";
		-- add the user itself to the list by default
		rewatch_AddPlayer(UnitName("player"));
		-- trigger the cooldown overlays
		for s,val in pairs(rewatch_abs) do rewatch_TriggerCooldown(s); end;
	-- else, if it's not empty, trigger a cooldown
	elseif(rewatch_update ~= "") then
		-- trigger the cooldown overlay
		rewatch_TriggerCooldown(rewatch_update);
		-- clear the update var
		rewatch_update = "";
	-- else, for each set of player data
	else for i,val in ipairs(rewatch_bars) do
		-- update player health bars if possible
		local a, b = val["PlayerBar"]:GetMinMaxValues(); if(b > 0) then
			-- get health data
			a, b = UnitHealthMax(val["Player"]), UnitHealth(val["Player"]);
			-- set health data
			val["PlayerBar"]:SetMinMaxValues(0, a); val["PlayerBar"]:SetValue(b);
			-- clear buffs if the player died (set it to 1 so it will be seen as expired)
			if((b <= 0) and (a > 0)) then val["Rejuvenation"], val["Regrowth"] = 1, 1;
			else
				-- set healthbar color accordingly
				if(b/a < .25) then val["PlayerBar"]:SetStatusBarColor(0.6, 0.0, 0.0, 0.9);
				elseif(a == b) then val["PlayerBar"]:SetStatusBarColor(0.0, 0.8, 0.0, 0.9);
				else val["PlayerBar"]:SetStatusBarColor(0.0, 0.6, 0.0, 0.8); end;
				-- check for poison and set the button accordingly
				if(rewatch_IsPoisoned(i)) then val["AbolishPoisonButton"]:SetNormalTexture("Interface\\Icons\\Spell_Shadow_VampiricAura.blp");
				else val["AbolishPoisonButton"]:SetNormalTexture("Interface\\Icons\\Spell_Nature_NullifyPoison_02.blp"); end;
			end;
		end;
		-- if no rejuvenation was cast on this player
		if(val["Rejuvenation"] == 0) then -- nuffin!
		-- else, if it has just expired
		elseif(GetTime() >= val["Rejuvenation"]) then
			-- clear the end time and bar value
			val["RejuvenationBar"]:SetValue(0); val["Rejuvenation"] = 0;
			-- if this player is not in the group and has no other HoTs on him, remove him from the pane (hide)
			if(val["Regrowth"] == 0) then if(not rewatch_InGroup(val["Player"])) then rewatch_HidePlayer(i); end; end;
		-- else update the bar's value
		else val["RejuvenationBar"]:SetValue(val["Rejuvenation"]-GetTime()); end;
		-- if no regrowth was cast on this player
		if(val["Regrowth"] == 0) then --nuffin!
		-- else, if it has just expired
		elseif(GetTime() >= val["Regrowth"]) then
			-- clear the end time and bar value
			val["RegrowthBar"]:SetValue(0); val["Regrowth"] = 0;
			-- if this player is not in the group and has no other HoTs on him, remove him from the pane (hide)
			if(val["Rejuvenation"] == 0) then if(not rewatch_InGroup(val["Player"])) then rewatch_HidePlayer(i); end; end;
		-- else update the bar's value
		else val["RegrowthBar"]:SetValue(val["Regrowth"]-GetTime()); end;
	end; end;
	
	-- return
	return;
end);
